// Code generated by go-swagger; DO NOT EDIT.

//
// Copyright 2021 The Sigstore Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package entries

import (
	"net/http"

	"github.com/go-openapi/runtime/middleware"
)

// CreateLogEntryHandlerFunc turns a function with the right signature into a create log entry handler
type CreateLogEntryHandlerFunc func(CreateLogEntryParams) middleware.Responder

// Handle executing the request and returning a response
func (fn CreateLogEntryHandlerFunc) Handle(params CreateLogEntryParams) middleware.Responder {
	return fn(params)
}

// CreateLogEntryHandler interface for that can handle valid create log entry params
type CreateLogEntryHandler interface {
	Handle(CreateLogEntryParams) middleware.Responder
}

// NewCreateLogEntry creates a new http.Handler for the create log entry operation
func NewCreateLogEntry(ctx *middleware.Context, handler CreateLogEntryHandler) *CreateLogEntry {
	return &CreateLogEntry{Context: ctx, Handler: handler}
}

// CreateLogEntry swagger:route POST /api/v1/log/entries entries createLogEntry
//
// # Creates an entry in the transparency log
//
// Creates an entry in the transparency log for a detached signature, public key, and content.
type CreateLogEntry struct {
	Context *middleware.Context
	Handler CreateLogEntryHandler
}

func (o *CreateLogEntry) ServeHTTP(rw http.ResponseWriter, r *http.Request) {
	route, rCtx, _ := o.Context.RouteInfo(r)
	if rCtx != nil {
		*r = *rCtx
	}
	params := NewCreateLogEntryParams()
	if err := o.Context.BindValidRequest(r, route, &params); err != nil { // bind params
		o.Context.Respond(rw, r, route.Produces, route, err)
		return
	}

	res := o.Handler.Handle(params) // actually handle the request

	o.Context.Respond(rw, r, route.Produces, route, res)

}
