// Code generated by go-swagger; DO NOT EDIT.

//
// Copyright 2021 The Sigstore Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package client

import (
	"maps"

	"github.com/go-openapi/runtime"
	httptransport "github.com/go-openapi/runtime/client"
	"github.com/go-openapi/strfmt"
	"github.com/sigstore/rekor/pkg/generated/client/entries"
	"github.com/sigstore/rekor/pkg/generated/client/index"
	"github.com/sigstore/rekor/pkg/generated/client/pubkey"
	"github.com/sigstore/rekor/pkg/generated/client/tlog"
)

// Default rekor HTTP client.
var Default = NewHTTPClient(nil)

const (
	// DefaultHost is the default Host found in Meta (info) section of spec file.
	DefaultHost string = "rekor.sigstore.dev"
	// DefaultBasePath is the default BasePath found in Meta (info) section of spec file.
	DefaultBasePath string = "/"
)

// DefaultSchemes are the default schemes found in Meta (info) section of spec file.
var DefaultSchemes = []string{"http"}

// NewHTTPClient creates a new rekor HTTP client.
func NewHTTPClient(formats strfmt.Registry) *Rekor {
	return NewHTTPClientWithConfig(formats, nil)
}

// NewHTTPClientWithConfig creates a new rekor HTTP client,
// using a customizable transport config.
func NewHTTPClientWithConfig(formats strfmt.Registry, cfg *TransportConfig) *Rekor {
	// ensure nullable parameters have default
	if cfg == nil {
		cfg = DefaultTransportConfig()
	}

	// create transport and client.
	transport := httptransport.New(cfg.Host, cfg.BasePath, cfg.Schemes)
	maps.Copy(transport.Producers, cfg.Producers)
	maps.Copy(transport.Consumers, cfg.Consumers)

	return New(transport, formats)
}

// New creates a new rekor client.
func New(transport runtime.ContextualTransport, formats strfmt.Registry) *Rekor {
	// ensure nullable parameters have default
	if formats == nil {
		formats = strfmt.Default
	}

	cli := new(Rekor)
	cli.Transport = transport
	cli.Entries = entries.New(transport, formats)
	cli.Index = index.New(transport, formats)
	cli.Pubkey = pubkey.New(transport, formats)
	cli.Tlog = tlog.New(transport, formats)

	return cli
}

// DefaultTransportConfig creates a TransportConfig with the
// default settings taken from the meta section of the spec file.
func DefaultTransportConfig() *TransportConfig {
	return &TransportConfig{
		Host:     DefaultHost,
		BasePath: DefaultBasePath,
		Schemes:  DefaultSchemes,
	}
}

// TransportConfig contains the transport related info,
// found in the meta section of the spec file.
type TransportConfig struct {
	Host      string
	BasePath  string
	Schemes   []string
	Producers map[string]runtime.Producer
	Consumers map[string]runtime.Consumer
}

// WithHost overrides the default host,
// provided by the meta section of the spec file.
func (cfg *TransportConfig) WithHost(host string) *TransportConfig {
	cfg.Host = host
	return cfg
}

// WithBasePath overrides the default basePath,
// provided by the meta section of the spec file.
func (cfg *TransportConfig) WithBasePath(basePath string) *TransportConfig {
	cfg.BasePath = basePath
	return cfg
}

// WithSchemes overrides the default schemes,
// provided by the meta section of the spec file.
func (cfg *TransportConfig) WithSchemes(schemes []string) *TransportConfig {
	cfg.Schemes = schemes
	return cfg
}

// WithProducers overrides the default producers registered by [httptransport.Runtime].
func (cfg *TransportConfig) WithProducers(producers map[string]runtime.Producer) *TransportConfig {
	cfg.Producers = producers
	return cfg
}

// WithConsumers overrides the default consumers registered by [httptransport.Runtime].
func (cfg *TransportConfig) WithConsumers(consumers map[string]runtime.Consumer) *TransportConfig {
	cfg.Consumers = consumers
	return cfg
}

// Rekor is a client for rekor.
type Rekor struct {
	Entries entries.ClientService

	Index index.ClientService

	Pubkey pubkey.ClientService

	Tlog tlog.ClientService

	Transport runtime.ContextualTransport
}

// SetTransport changes the transport on the client and all its subresources.
func (c *Rekor) SetTransport(transport runtime.ContextualTransport) {
	c.Transport = transport
	c.Entries.SetTransport(transport)
	c.Index.SetTransport(transport)
	c.Pubkey.SetTransport(transport)
	c.Tlog.SetTransport(transport)
}
